def action_print():
    print('It works!')


def print_ok():
    print('ok')


def print_bad():
    print('bad')


def print_flag_force(force: bool):
    print(f'force: {force}')


def print_param(param):
    print(f'param: {param}')


def list_devices():
    return ['sda', 'sdb', 'sdc']
